__author__ = 'DarkWeb'

# Here, we are importing the auxiliary functions to clean or convert data
from MarketPlaces.Utilities.utilities import *

# Here, we are importing BeautifulSoup to search through the HTML tree
from bs4 import BeautifulSoup, ResultSet, Tag


def anonymousMarketplace_description_parser(soup: Tag):
    
    # Fields to be parsed

    vendor = "-1"                       # 0 *Vendor_Name
    success = "-1"                      # 1 Vendor_Successful_Transactions
    rating_vendor = "-1"                # 2 Vendor_Rating
    name = "-1"                         # 3 *Product_Name
    describe = "-1"                     # 4 Product_Description
    CVE = "-1"                          # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = "-1"                           # 6 Product_MS_Classification (Microsoft Security)
    category = "-1"                     # 7 Product_Category
    views = "-1"                        # 8 Product_Number_Of_Views
    reviews = "-1"                      # 9 Product_Number_Of_Reviews
    rating_item = "-1"                  # 10 Product_Rating
    addDate = "-1"                      # 11 Product_AddedDate
    BTC = "-1"                          # 12 Product_BTC_SellingPrice
    USD = "-1"                          # 13 Product_USD_SellingPrice
    EURO = "-1"                         # 14 Product_EURO_SellingPrice
    sold = "-1"                         # 15 Product_QuantitySold
    left = "-1"                         # 16 Product_QuantityLeft
    shipFrom = "-1"                     # 17 Product_ShippedFrom
    shipTo = "-1"                       # 18 Product_ShippedTo
    
    product_name = soup.find("h1", {"class": "product_title entry-title"}).text
    name = cleanString(product_name.strip())
    
    product_description_list: ResultSet[Tag] = soup.find("div", {"id": "tab-description"}).find_all("div")
    describe_output = ""
    
    for div in product_description_list:
        describe_output += div.text
    describe = cleanString(describe_output.strip())
    
    product_ratings: Tag = soup.find("div", {"class": "star-rating"})
    
    product_reviews = product_ratings.find("div", {"class": "woocommerce-product-rating"}).find("strong", {"class": "rating"}).text
    reviews = cleanString(product_reviews.strip())
    
    product_star_rating = product_ratings.find("span", {"class": "rating"}).text
    rating_item = cleanString(product_star_rating.strip())
    
    product_price = soup.find("span", {"class": "woocommerce-Price-amount amount"}).text
    USD = cleanString(product_price.replace("$", "").strip())
    
    
    # Populating the final variable (this should be a list with all fields scraped)
    row = (vendor, rating_vendor, success, name, describe, CVE, MS, category, views, reviews, rating_item, addDate,
           BTC, USD, EURO, sold, left, shipFrom, shipTo)

    # Sending the results
    return row


def anonymousMarketplace_listing_parser(soup: Tag):
    
    # Fields to be parsed
    nm = 0                                    # *Total_Products (Should be Integer)
    mktName = "AnonymousMarketplace"          # 0 *Marketplace_Name
    vendor = []                               # 1 *Vendor y
    rating_vendor = []                        # 2 Vendor_Rating
    success = []                              # 3 Vendor_Successful_Transactions
    name = []                                 # 4 *Product_Name y
    CVE = []                                  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = []                                   # 6 Product_MS_Classification (Microsoft Security)
    category = []                             # 7 Product_Category y
    describe = []                             # 8 Product_Description
    views = []                                # 9 Product_Number_Of_Views
    reviews = []                              # 10 Product_Number_Of_Reviews
    rating_item = []                          # 11 Product_Rating
    addDate = []                              # 12 Product_AddDate
    BTC = []                                  # 13 Product_BTC_SellingPrice
    USD = []                                  # 14 Product_USD_SellingPrice y
    EURO = []                                 # 15 Product_EURO_SellingPrice
    sold = []                                 # 16 Product_QuantitySold
    qLeft =[]                                 # 17 Product_QuantityLeft
    shipFrom = []                             # 18 Product_ShippedFrom
    shipTo = []                               # 19 Product_ShippedTo
    href = []                                 # 20 Product_Links
    
    
    product_list: ResultSet[Tag] = soup.find("ul", {"class": "products columns-4"}).find_all("li")
    
    for item in product_list:
        item_href = item.find("a").get("href")
        href.append(item_href)
        
        item_name = item.find("h2", {"class": "woocommerce-loop-product__title"}).text
        name.append(cleanString('item_name'.strip()))
        
        item_rating = item.find("div", {"class": "star-rating"}).find("strong", {"class": "rating"}).text
        rating_item.append(cleanString(item_rating.strip()))
        
        try:
            item_price = item.find("span", {"class": "woocommerce-Price-amount amount"}).text
            item_price = item_price.replace("$", "").strip()
            USD.append(item_price)
        except AttributeError:
            USD.append("-1")
        
        
            
        
        vendor.append("Anonymous")
        rating_vendor.append("-1")
        success.append("-1")
        CVE.append("-1")
        MS.append("-1")
        category.append("-1")
        describe.append("-1")
        views.append("-1")
        reviews.append("-1")
        addDate.append("-1")
        BTC.append("-1")
        EURO.append("-1")
        sold.append("-1")
        qLeft.append("-1")
        shipFrom.append("-1")
        shipTo.append("-1")
        
        nm += 1
    
    return organizeProducts(
        marketplace=mktName,
        nm=nm,
        vendor=vendor,
        rating_vendor=rating_vendor,
        success_vendor=success,
        nombre=name,
        CVE=CVE,
        MS=MS,
        category=category,
        describe=describe,
        views=views,
        reviews=reviews,
        rating_item=rating_item,
        addDate=addDate,
        BTC=BTC,
        USD=USD,
        EURO=EURO,
        sold=sold,
        qLeft=qLeft,
        shipFrom=shipFrom,
        shipTo=shipTo,
        href=href
    )
    
        
        
        


#called by the crawler to get description links on a listing page
#@param: beautifulsoup object that is using the correct html page (listing page)
#return: list of description links from a listing page
def anonymous_links_parser(soup):

    # Returning all links that should be visited by the Crawler

    href = []
    listing = soup.find('ul', {"class": "products columns-4"}).findAll('li')

    for a in listing:
        bae = a.find('a', {"class": "woocommerce-LoopProduct-link woocommerce-loop-product__link"}, href=True)

        link = bae['href']
        href.append(link)

    return href