__author__ = 'DarkWeb'

# Here, we are importing the auxiliary functions to clean or convert data
from MarketPlaces.Utilities.utilities import *

# Here, we are importing BeautifulSoup to search through the HTML tree
from bs4 import BeautifulSoup, ResultSet, Tag


#parses description pages, so takes html pages of description pages using soup object, and parses it for info it needs
#stores info it needs in different lists, these lists are returned after being organized
#@param: soup object looking at html page of description page
#return: 'row' that contains a variety of lists that each hold info on the description page
def digitalThriftShop_description_parser(soup: Tag):

    # Fields to be parsed

    vendor = "-1"                       # 0 *Vendor_Name
    success = "-1"                      # 1 Vendor_Successful_Transactions
    rating_vendor = "-1"                # 2 Vendor_Rating
    name = "-1"                         # 3 *Product_Name
    describe = "-1"                     # 4 Product_Description
    CVE = "-1"                          # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = "-1"                           # 6 Product_MS_Classification (Microsoft Security)
    category = "-1"                     # 7 Product_Category
    views = "-1"                        # 8 Product_Number_Of_Views
    reviews = "-1"                      # 9 Product_Number_Of_Reviews
    rating_item = "-1"                  # 10 Product_Rating
    addDate = "-1"                      # 11 Product_AddedDate
    BTC = "-1"                          # 12 Product_BTC_SellingPrice
    USD = "-1"                          # 13 Product_USD_SellingPrice
    EURO = "-1"                         # 14 Product_EURO_SellingPrice
    sold = "-1"                         # 15 Product_QuantitySold
    left = "-1"                         # 16 Product_QuantityLeft
    shipFrom = "-1"                     # 17 Product_ShippedFrom
    shipTo = "-1"                       # 18 Product_ShippedTo



    product_name = soup.find("h1", {"class": "product_title entry-title"}).text
    name = cleanString(product_name.strip())

    product_description = soup.find("div", {"id": "tab-description"}).find("p").text
    describe = cleanString(product_description.strip())
    
    product_category = soup.find("span", {"class": "posted_in"}).find("a").text
    category = cleanString(product_category.strip())
    
    try:
        product_rating: Tag = soup.find("div", {"class": "woocommerce-product-rating"})
        rating_item = product_rating.find("strong", {"class": "rating"}).text
        reviews = product_rating.find("span", {"Class": "rating"}).text
    
    except Exception as e:
        pass

    product_BTC = soup.find("div", {"id": "price-BTC"}).find("span", {"class": "priceinfo cw-noselect"}).text
    BTC = cleanString(product_BTC.strip())

    product_USD = soup.find("span", {"class": "woocommerce-Price-amount amount"}).text
    USD = cleanString(product_USD.replace("$", "").strip())
    
    

    # Populating the final variable (this should be a list with all fields scraped)
    row = (vendor, rating_vendor, success, name, describe, CVE, MS, category, views, reviews, rating_item, addDate,
           BTC, USD, EURO, sold, left, shipFrom, shipTo)

    # Sending the results
    return row


#parses listing pages, so takes html pages of listing pages using soup object, and parses it for info it needs
#stores info it needs in different lists, these lists are returned after being organized
#@param: soup object looking at html page of listing page
#return: 'row' that contains a variety of lists that each hold info on the listing page
def digitalThriftShop_listing_parser(soup: Tag):

   # Fields to be parsed
    nm = 0                                    # *Total_Products (Should be Integer)
    mktName = "DigitalThriftShop"             # 0 *Marketplace_Name
    vendor = []                               # 1 *Vendor y
    rating_vendor = []                        # 2 Vendor_Rating
    success = []                              # 3 Vendor_Successful_Transactions
    name = []                                 # 4 *Product_Name y
    CVE = []                                  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = []                                   # 6 Product_MS_Classification (Microsoft Security)
    category = []                             # 7 Product_Category y
    describe = []                             # 8 Product_Description
    views = []                                # 9 Product_Number_Of_Views
    reviews = []                              # 10 Product_Number_Of_Reviews
    rating_item = []                          # 11 Product_Rating
    addDate = []                              # 12 Product_AddDate
    BTC = []                                  # 13 Product_BTC_SellingPrice
    USD = []                                  # 14 Product_USD_SellingPrice y
    EURO = []                                 # 15 Product_EURO_SellingPrice
    sold = []                                 # 16 Product_QuantitySold
    qLeft =[]                                 # 17 Product_QuantityLeft
    shipFrom = []                             # 18 Product_ShippedFrom
    shipTo = []                               # 19 Product_ShippedTo
    href = []                                 # 20 Product_Links

    product_category = soup.find("h1", {"class": "woocommerce-products-header__title page-title"}).text

    products_list: ResultSet[Tag] = soup.find("ul", {"class": "products columns-5"}).find_all("li")
    
    
    
    for product in products_list:
        nm += 1
        vendor.append("-1")
        rating_vendor.append("-1")
        success.append("-1")
        
        product_name = product.find("h2", {"class": "woocommerce-loop-product__title"}).text
        name.append(cleanString(product_name.strip()))
        
        CVE.append("-1")
        MS.append("-1")
        category.append(cleanString(product_category.strip()))
        describe.append("-1")
        views.append("-1")
        reviews.append("-1")
        
        try:
            product_rating = product.find("div", {"class": "star-rating"}).find("strong", {"class": "rating"}).text
            rating_item.append(cleanString(product_rating.strip()))
        except:
            rating_item.append("-1")
        
        addDate.append("-1")
        BTC.append("-1")
        
        product_USD = product.find("span", {"class": "price"}).text
        USD.append(product_USD.replace("$", "").strip())
        
        EURO.append("-1")
        sold.append("-1")
        qLeft.append("-1")
        shipFrom.append("-1")
        shipTo.append("-1")

        product_href = product.find("a", {"class": "woocommerce-LoopProduct-link woocommerce-loop-product__link"}).get("href")
        href.append(cleanString(product_href.strip()))
    

    # Populate the final variable (this should be a list with all fields scraped)
    return organizeProducts(mktName, nm, vendor, rating_vendor, success, name, CVE, MS, category, describe, views,
                            reviews, rating_item, addDate, BTC, USD, EURO, sold, qLeft, shipFrom, shipTo, href)


#called by the crawler to get description links on a listing page
#@param: beautifulsoup object that is using the correct html page (listing page)
#return: list of description links from a listing page
def digitalthriftshop_links_parser(soup):

    # Returning all links that should be visited by the Crawler

    href = []
    listing = soup.find('ul', {"class": "products columns-5"}).findAll('li')

    for a in listing:
        bae = a.find('a', href=True)
        link = bae['href']
        href.append(link)

    return href