__author__ = 'DarkWeb'

# Here, we are importing the auxiliary functions to clean or convert data
from MarketPlaces.Utilities.utilities import *

# Here, we are importing BeautifulSoup to search through the HTML tree
from bs4 import BeautifulSoup

import re

#parses description pages, so takes html pages of description pages using soup object, and parses it for info it needs
#stores info it needs in different lists, these lists are returned after being organized
#@param: soup object looking at html page of description page
#return: 'row' that contains a variety of lists that each hold info on the description page
def tormarket_description_parser(soup):

    # Fields to be parsed

    name = "-1"                         # 0 Product_Name
    describe = "-1"                     # 1 Product_Description
    lastSeen = "-1"                     # 2 Product_LastViewDate
    rules = "-1"                        # 3 NOT USED ...
    CVE = "-1"                          # 4 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = "-1"                           # 5 Product_MS_Classification (Microsoft Security)
    review = "-1"                       # 6 Product_Number_Of_Reviews
    category = "-1"                     # 7 Product_Category
    shipFrom = "-1"                     # 8 Product_ShippedFrom
    shipTo = "-1"                       # 9 Product_ShippedTo
    left = "-1"                         # 10 Product_QuantityLeft
    escrow = "-1"                       # 11 Vendor_Warranty
    terms = "-1"                        # 12 Vendor_TermsAndConditions
    vendor = "-1"                       # 13 Vendor_Name
    sold = "-1"                         # 14 Product_QuantitySold
    addDate = "-1"                      # 15 Product_AddedDate
    available = "-1"                    # 16 NOT USED ...
    endDate = "-1"                      # 17 NOT USED ...
    BTC = "-1"                          # 18 Product_BTC_SellingPrice
    USD = "-1"                          # 19 Product_USD_SellingPrice
    rating = "-1"                       # 20 Vendor_Rating
    success = "-1"                      # 21 Vendor_Successful_Transactions
    EURO = "-1"                         # 22 Product_EURO_SellingPrice

    #finding the name of the product
    name_of_product = soup.find("h1", {"class": "product_title entry-title"}).find("a").text
    name = cleanString(name_of_product.strip())
    #finding the description of the product
    description_of_product = soup.find("div", {"class": "woocommerce-product-details__short-description"}).text
    describe = cleanString(description_of_product.strip())
    #finding the replies
    inquires_about_product = soup.find("div", {"class": "woocommerce-Tabs-panel woocommerce-Tabs-panel--wcfm_enquiry_tab panel entry-content wc-tab"}).find("p").text
    if inquires_about_product == "There are no inquiries yet.":
        review = 0
    else:
        review = -1 #fix later pls

    #finding the terms and conditions
    terms_and_conditions = soup.find("div", {"class": "woocommerce-Tabs-panel woocommerce-Tabs-panel--wcfm_enquiry_tab panel entry-content wc-tab"}).find("p").text
    term = cleanString(terms_and_conditions)

    #finding the name of the vendor
    name_of_vendor = soup.find("div", {"class": "wcfmmp_sold_by_store"}).find("a").text
    vendor = cleanString(name_of_vendor)

    #finding the price of the item
    price = soup.find("p", {"class": "price"}).find("bdi").text
    price_cleaned = price[1:]
    USD = price_cleaned.strip()
    #everything else gets a -1 because they are not found

    # Populating the final variable (this should be a list with all fields scraped)
    row = (name, describe, lastSeen, rules, CVE, MS, review, category, shipFrom, shipTo, left, escrow, terms, vendor,
       sold, addDate, available, endDate, BTC, USD, rating, success, EURO)

    # Sending the results
    return row


#parses listing pages, so takes html pages of listing pages using soup object, and parses it for info it needs
#stores info it needs in different lists, these lists are returned after being organized
#@param: soup object looking at html page of listing page
#return: 'row' that contains a variety of lists that each hold info on the listing page
def tormarket_listing_parser(soup):

    # Fields to be parsed
    nm = 0  # *Total_Products (Should be Integer)
    mktName = "TorMarket"  # 0 *Marketplace_Name
    vendor = []  # 1 *Vendor y
    rating_vendor = []  # 2 Vendor_Rating
    success = []  # 3 Vendor_Successful_Transactions
    name = []  # 4 *Product_Name y
    CVE = []  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = []  # 6 Product_MS_Classification (Microsoft Security)
    category = []  # 7 Product_Category y
    describe = []  # 8 Product_Description
    views = []  # 9 Product_Number_Of_Views
    reviews = []  # 10 Product_Number_Of_Reviews
    rating_item = []  # 11 Product_Rating
    addDate = []  # 12 Product_AddDate
    BTC = []  # 13 Product_BTC_SellingPrice
    USD = []  # 14 Product_USD_SellingPrice y
    EURO = []  # 15 Product_EURO_SellingPrice
    sold = []  # 16 Product_QuantitySold
    qLeft = []  # 17 Product_QuantityLeft
    shipFrom = []  # 18 Product_ShippedFrom
    shipTo = []  # 19 Product_ShippedTo
    href = []  # 20 Product_Links
    products_list = soup.find_all('li')
    nm = 0
    for product in products_list:
        try:
            # Finding the name of the product
            name_of_product = product.find("h2", {"class": "woocommerce-loop-product__title"}).find("a").text
            name_of_product_cleaned = cleanString(name_of_product.strip())
            print(name_of_product_cleaned)
            name.append(name_of_product_cleaned)
            #finding the URL
            try:
                url = product.find("div", {"class": "product-loop-content text-center"}).find("a").get("href")
                print(url)
                href.append(url)
            except AttributeError as e:
                print("I can't find the link")
                raise e

            #finding the rating of the product
            rating_score_of_product = product.find("div", {"class": "product-loop-content text-center"}).find("div").find("span").text
            rating_item.append(cleanString(rating_score_of_product.strip()))
            print("done")
            #finding the rating of the vendors
            rating_score_of_vendor = product.find("div", {"class": "wcfmmp-store-rating"}).find("strong").text
            rating_vendor.append(cleanString(rating_score_of_vendor.strip()))
            print("done")
            #finding the cost in USD
            cost = product.find("span", {"class": "woocommerce-Price-amount amount"}).text
            USD.append(cost)
            print("done")
            #finding the name of the vendor
            vendor_name = product.find("div", {"class": "wcfmmp_sold_by_wrapper"}).find("a").text
            vendor.append(cleanString(vendor_name.strip()))
            print("done")
            #everything else appends a -1
            success.append("-1")
            CVE.append("-1")
            MS.append("-1")
            category.append("-1")
            describe.append("-1")
            views.append("-1")
            reviews.append("-1")
            addDate.append("-1")
            BTC.append("-1")
            EURO.append("-1")
            sold.append("-1")
            qLeft.append("-1")
            shipFrom.append("-1")
            shipTo.append("-1")
            print("Done! moving onto the next product!")
            print(len(shipTo))
            nm += 1
        except AttributeError as e:
            print("I'm somewhere I don't belong. I'm going to leave")
            continue


    # Populate the final variable (this should be a list with all fields scraped)
    return organizeProducts(
        marketplace = "TorMarket",
        nm = nm,
        vendor = vendor,
        rating_vendor = rating_vendor,
        success_vendor = success,
        nombre = name,
        CVE = CVE,
        MS = MS,
        category = category,
        describe = describe,
        views = views,
        reviews = reviews,
        rating_item = rating_item,
        addDate = addDate,
        BTC = BTC,
        USD = USD,
        EURO = EURO,
        sold = sold,
        qLeft = qLeft,
        shipFrom = shipFrom,
        shipTo = shipTo,
        href = href
    )


#called by the crawler to get description links on a listing page
#@param: beautifulsoup object that is using the correct html page (listing page)
#return: list of description links from a listing page
def tormarket_links_parser(soup):

    # Returning all links that should be visited by the Crawler

    href = []
    listing = soup.findAll('div', {"class": "product-loop-content text-center"})

    for a in listing:
        bae = a.find('h2', {"class": "woocommerce-loop-product__title"}).find('a', href=True)
        link = bae['href']
        href.append(link)

    return href