__author__ = 'DarkWeb'

import re

# Here, we are importing the auxiliary functions to clean or convert data
from MarketPlaces.Utilities.utilities import *

# Here, we are importing BeautifulSoup to search through the HTML tree
from bs4 import BeautifulSoup


# This is the method to parse the Description Pages (one page to each Product in the Listing Pages)
def vicecity_description_parser(soup):
    # Fields to be parsed

    vendor = "-1"  # 0 *Vendor_Name
    success = "-1"  # 1 Vendor_Successful_Transactions
    rating_vendor = "-1"  # 2 Vendor_Rating
    name = "-1"  # 3 *Product_Name
    describe = "-1"  # 4 Product_Description
    CVE = "-1"  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = "-1"  # 6 Product_MS_Classification (Microsoft Security)
    category = "-1"  # 7 Product_Category
    views = "-1"  # 8 Product_Number_Of_Views
    reviews = "-1"  # 9 Product_Number_Of_Reviews
    rating_item = "-1"  # 10 Product_Rating
    addDate = "-1"  # 11 Product_AddedDate
    BTC = "-1"  # 12 Product_BTC_SellingPrice
    USD = "-1"  # 13 Product_USD_SellingPrice
    EURO = "-1"  # 14 Product_EURO_SellingPrice
    sold = "-1"  # 15 Product_QuantitySold
    left = "-1"  # 16 Product_QuantityLeft
    shipFrom = "-1"  # 17 Product_ShippedFrom
    shipTo = "-1"  # 18 Product_ShippedTo

    # Finding Product Name
    name = soup.find('div', {'class': "listing_info"}).find('div', {'class': "listing_right"})
    name = name.find('span', {'style': "font-size:18px;font-weight: bold;color: #fff"}).text
    name = name.replace('\n', ' ')
    name = name.replace(",", "")
    name = name.strip()

    # Finding Vendor
    vendor = soup.find('div', {'class': "listing_info"})
    vendor = vendor.find('div', {'class': "listing_right"})
    numbers = vendor.find('a').find('span').text
    vendor = vendor.find('a').text
    vendor = vendor.replace(numbers, "").strip() # removes numbers at the end of vendor name

    # Finding Vendor Rating
    rating = soup.find('div', {'class': "listing_info"}).find('div', {'class': "listing_right"}).find('a').get('title')
    rating = re.search(r"\d+%", rating)
    rating_vendor = rating.group(0).strip()

    # Finding Quantity Sold and Left
    # temp = mb[4].text.split(',')
    #
    # sold = temp[0].replace("sold", "")
    # sold = sold.strip()
    #
    # left = temp[1].replace("in stock", "")
    # left = left.strip()

    # Finding Successful Transactions
    success = soup.find('div', {'class': "listing_info"}).find('div', {'class': "listing_right"}).find('a').get('title')
    success = re.search(r"\d+(?= sales)", success)
    success = success.group(0).strip()


    bae = soup.find('pre')

    # Finding USD
    USD = bae.find('span').text
    USD = re.search(r"\$\d+(?:\.\d+)?", USD).group(0)
    USD = USD.replace("$", "").strip()

    # Finding BTC
    BTC = bae.find_all('span')
    BTC = re.search(r"\d+(?:\.\d+)?", BTC[1].text).group(0)
    BTC = BTC.strip()

    # Finding the Product Category
    category = soup.find('div', {'class': "listing_info"}).find('div', {'class': "listing_right"})
    category = category.find('span', {'style': "font-size:15px;color: #a1a1a1"}).text
    category = category.replace("Category:", "").strip()

    li = bae.find_all('span')

    # Finding Shipment Information (Origin)
    shipFrom = li[-4].text.strip()

    # Finding Shipment Information (Destination)
    shipTo = li[-2].text.strip()

    # Finding the Product description
    describe = soup.find('p', {
        'style': "width:705px;margin-left:-305px;background-color: #242424;border-radius: 3px;border: 1px solid #373737;padding: 5px;"}).text
    describe = describe.replace("\n", " ")
    describe = describe.strip()

    # Finding the Number of Product Reviews
    li = soup.find_all('label', {'class': "tc_label threetabs"})
    review = li[1].text
    review = re.search(r"\d+", review)
    if review:
        reviews = review.group(0).strip()
    else:
        reviews = '0'

    # Searching for CVE and MS categories
    cve = soup.findAll(text=re.compile('CVE-\d{4}-\d{4}'))
    if cve:
        CVE = " "
        for idx in cve:
            CVE += (idx)
            CVE += "  "
            CVE = CVE.replace(',', ' ')
            CVE = CVE.replace('\n', '')
    ms = soup.findAll(text=re.compile('MS\d{2}-\d{3}'))
    if ms:
        MS = " "
        for im in ms:
            MS += (im)
            MS += " "
            MS = MS.replace(',', ' ')
            MS = MS.replace('\n', '')

    # Populating the final variable (this should be a list with all fields scraped)
    row = (vendor, rating_vendor, success, name, describe, CVE, MS, category, views, reviews, rating_item, addDate,
           BTC, USD, EURO, sold, left, shipFrom, shipTo)

    # Sending the results
    return row


# This is the method to parse the Listing Pages
def vicecity_listing_parser(soup):
    # Fields to be parsed
    nm = 0  # *Total_Products (Should be Integer)
    mktName = "ViceCity"  # 0 *Marketplace_Name
    vendor = []  # 1 *Vendor y
    rating_vendor = []  # 2 Vendor_Rating
    success = []  # 3 Vendor_Successful_Transactions
    name = []  # 4 *Product_Name y
    CVE = []  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = []  # 6 Product_MS_Classification (Microsoft Security)
    category = []  # 7 Product_Category y
    describe = []  # 8 Product_Description
    views = []  # 9 Product_Number_Of_Views
    reviews = []  # 10 Product_Number_Of_Reviews
    rating_item = []  # 11 Product_Rating
    addDate = []  # 12 Product_AddDate
    BTC = []  # 13 Product_BTC_SellingPrice
    USD = []  # 14 Product_USD_SellingPrice y
    EURO = []  # 15 Product_EURO_SellingPrice
    sold = []  # 16 Product_QuantitySold
    qLeft = []  # 17 Product_QuantityLeft
    shipFrom = []  # 18 Product_ShippedFrom
    shipTo = []  # 19 Product_ShippedTo
    href = []  # 20 Product_Links

    listing = soup.findAll('div', {"class": "wLf"}) # should be 30

    # Populating the Number of Products
    nm = len(listing)

    # # Finding Category
    # cat = soup.find("div", {"class": "col-9"})
    # cat = cat.find("h2").text
    # cat = cat.replace("Category: ", "")
    # cat = cat.replace(",", "")
    # cat = cat.strip()

    for a in listing:
        # category.append(cat)

        # bae = card.findAll('a')

        # Adding the url to the list of urls
        link = a.find('div', {"class": "wLfLeft"}).find('a', href=True).get('href')
        link = cleanLink(link)
        href.append(link)

        # Finding the Product Name
        product = a.find('div', {"class": "wLfName"}).find('a').text
        product = product.replace('\n', ' ')
        product = product.replace(",", "")
        product = product.replace("...", "")
        product = product.strip()
        name.append(product)

        # Finding the Vendor
        vendor_name = a.find('div', {"class": "wLfVendor"}).find('a').text
        addedNums = a.find('div', {"class": "wLfVendor"}).find('a').find('span').text # finds numbers added at end
        vendor_name = vendor_name.replace(",", "")
        vendor_name = vendor_name.replace(addedNums, "") # removes numbers added at end
        vendor_name = vendor_name.strip()
        vendor.append(vendor_name)

        # Finding Prices
        price = a.find('div', {"class": "wLfPrice"}).find_all('span')
        ud = price[0].text.replace(" USD", " ")
        # u = ud.replace("$","")
        ud = ud.replace(",", "")
        u = ud.replace(price[1].text, "")
        u = u.strip()
        USD.append(u)
        bc = price[1].text
        bc = re.search(r"\d+(?:\.\d+)?", bc).group(0).strip()
        BTC.append(bc)

        # # Finding Reviews
        # num = card.find("span", {"class": "rate-count"}).text
        # num = num.replace("(", "")
        # num = num.replace("review)", "")
        # num = num.replace("reviews)", "")
        # num = num.strip()
        # reviews.append(num)

        # Finding Successful Transactions
        freq = a.find('div', {"class": "wLfVendor"}).find('a').get('title')
        freq = re.search(r'\d+(?= sales)', freq).group(0)
        freq = freq.strip()
        success.append(freq)

        # Finding Ship from and ship to
        place = a.find('div', {"class": "wLfPrice"})
        place = place.find('span', {'style': "font-size: 12px;"}).text
        place = place.split('⟶')
        varFrom = place[0].strip()
        varTo = place[1].strip()
        if varFrom == "WW":
            varFrom = "Worldwide"
        if varTo == "WW":
            varTo = "Worldwide"
        shipFrom.append(varFrom)
        shipTo.append(varTo)

        # Searching for CVE and MS categories
        cve = a.findAll(text=re.compile('CVE-\d{4}-\d{4}'))
        if not cve:
            cveValue = "-1"
        else:
            cee = " "
            for idx in cve:
                cee += (idx)
                cee += "  "
                cee = cee.replace(',', ' ')
                cee = cee.replace('\n', '')
            cveValue = cee
        CVE.append(cveValue)

        ms = a.findAll(text=re.compile('MS\d{2}-\d{3}'))
        if not ms:
            MSValue = "-1"
        else:
            me = " "
            for im in ms:
                me += (im)
                me += " "
                me = me.replace(',', ' ')
                me = me.replace('\n', '')
            MSValue = me
        MS.append(MSValue)

    # Populate the final variable (this should be a list with all fields scraped)
    return organizeProducts(mktName, nm, vendor, rating_vendor, success, name, CVE, MS, category, describe, views,
                            reviews, rating_item, addDate, BTC, USD, EURO, sold, qLeft, shipFrom, shipTo, href)


def vicecity_links_parser(soup):
    # Returning all links that should be visited by the Crawler

    href = []
    listing = soup.findAll('div', {"class": "wLf"})

    for a in listing:
        bae = a.find('div', {"class": "wLfLeft"}).find('a', href=True)
        link = bae['href']
        href.append(link)

    return href
